import typing as t

import pandas as pd

from model import __version__ as _version
from model.config.core import config
from model.processing.data_manager import load_pipeline
from model.processing.validation import validate_inputs
import logging

pipeline_file_name = f"{config.app_config.pipeline_save_file}{_version}.pkl"
_price_pipe = load_pipeline(filename=pipeline_file_name)


def make_prediction(
    *,
    input_data: t.Union[pd.DataFrame, dict],
) -> dict:
    """Make a prediction using a saved model pipeline."""

    data = pd.DataFrame(input_data)

    validated_data, errors = validate_inputs(input_data=data)
    results = {"predictions": None, "version": _version, "errors": errors}

    if not errors:
        predictions = _price_pipe.predict(
            X=validated_data[config.model_config.features]
        )

        results = {"predictions": [result for result in predictions],
                   "version": _version,
                   "errors": errors}

    return results



from sklearn.ensemble import RandomForestClassifier
from sklearn.pipeline import Pipeline
from sklearn.preprocessing import StandardScaler

from model.config.core import config

price_pipe = Pipeline(
    [
        # ===== Standard Scaler =====
        # Make the features to be in the same scale
        (
            "StandardScaler",
            StandardScaler(),
        ),
        (
            "rf",
            RandomForestClassifier(
                n_estimators=config.model_config.n_estimators,
                bootstrap=config.model_config.bootstrap,
                criterion=config.model_config.criterion,
                max_depth=config.model_config.max_depth,
                max_features=config.model_config.max_features,
                max_leaf_nodes=config.model_config.max_leaf_nodes,
                max_samples=config.model_config.max_samples,
                n_jobs=config.model_config.n_jobs,
            ),
        ),
    ]
)



from typing import List, Optional, Tuple

import numpy as np
import pandas as pd
from pydantic import BaseModel, ValidationError

from model.config.core import config


def drop_na_inputs(*, input_data: pd.DataFrame) -> pd.DataFrame:
    """Check model inputs for na values and filter."""
    validated_data = input_data.copy()
    new_vars_with_na = [
        var
        for var in config.model_config.features
        if validated_data[var].isnull().sum() > 0
    ]

    validated_data.dropna(subset=new_vars_with_na, inplace=True)

    return validated_data


def validate_inputs(*, input_data: pd.DataFrame) -> Tuple[pd.DataFrame, Optional[dict]]:
    """Check model inputs for unprocessable values."""

    relevant_data = input_data[config.model_config.features].copy()
    validated_data = drop_na_inputs(input_data=relevant_data)
    errors = None

    try:
        # replace numpy nans so that pydantic can validate
        MultipleMobileDataInputs(
            inputs=validated_data.replace({np.nan: None}).to_dict(orient="records")
        )
    except ValidationError as error:
        errors = error.json()

    return validated_data, errors


class MobileDataInputSchema(BaseModel):
    battery_power: Optional[int]
    blue: Optional[int]
    clock_speed: Optional[float]
    dual_sim: Optional[int]
    fc: Optional[int]
    four_g: Optional[int]
    int_memory: Optional[int]
    m_dep: Optional[float]
    mobile_wt: Optional[int]
    n_cores: Optional[int]
    pc: Optional[int]
    px_height: Optional[int]
    px_width: Optional[int]
    ram: Optional[int]
    sc_h: Optional[int]
    sc_w: Optional[int]
    talk_time: Optional[int]
    three_g: Optional[int]
    touch_screen: Optional[int]
    wifi: Optional[int]


class MultipleMobileDataInputs(BaseModel):
    inputs: List[MobileDataInputSchema]
